

import json

from django.conf import settings
from django.views.generic.base import View

from zmei.json import ZmeiReactJsonEncoder
from zmei.react import ZmeiReactServer


class _Data(object):
    def __init__(self, data=None):
        self.__dict__.update(data or {})

    def __add__(self, data):
        return _Data({**self.__dict__, **data})


class ZmeiDataViewMixin(View):
    _data = None

    def get_data(self, url, request, inherited):
        return {}

    def _get_data(self):
        if not self._data:
            self._data = self.get_data(
                url=type('url', (object,), self.kwargs),
                request=self.request,
                inherited=False
            )

        return self._data

    def get_context_data(self, **kwargs):
        context_data = super().get_context_data(**self.kwargs)

        return {**context_data, **self._get_data()}


class ImproperlyConfigured(Exception):
    pass


class ZmeiReactViewMixin(ZmeiDataViewMixin):

    react_server = None
    react_components = None

    def get_context_data(self, **kwargs):
        data = super().get_context_data(**kwargs)

        if not isinstance(self.react_server, ZmeiReactServer):
            raise ImproperlyConfigured('ZmeiReactViewMixin requires react_server property')

        if not isinstance(self.react_components, list):
            raise ImproperlyConfigured('ZmeiReactViewMixin requires react_component property')

        data['react_state'] = ZmeiReactJsonEncoder(view=self).encode(self._get_data())

        if settings.DEBUG:
            self.react_server.autreload()

        for cmp in self.react_components:
            data[f'react_page_{cmp}'] = self.react_server.evaljs(
                f"R.ReactDOM.renderToString(R.React.createElement(R.{cmp}, {data['react_state']}), null);")

        return data



import os
import re

from dukpy import JSInterpreter
from dukpy._dukpy import JSRuntimeError
from termcolor import colored


class ZmeiReactServer(object):
    def __init__(self):
        super().__init__()

        self.loaded_files = []
        self.loaded_files_mtime = {}

        self.jsi = None
        self.reload_interpreter()

        self.checksum = None

    def reload_interpreter(self):
        self.jsi = JSInterpreter()

        code = """
        Duktape.errCreate = function (err) {
            try {
                if (typeof err === 'object' &&
                    typeof err.message !== 'undefined' &&
                    typeof err.lineNumber === 'number') {
                    err.message = err.message + ' (line ' + err.lineNumber + ')';
                }
            } catch (e) {
                // ignore; for cases such as where "message" is not writable etc
            }
            return err;
        };

        var global = this;
        var module = {exports: {}};
        var setTimeout = function(){};
        var clearTimeout = function(){};
        """

        self.jsi.evaljs(code)

        for filename in self.loaded_files:
            self.loaded_files_mtime[filename] = os.path.getmtime(filename)
            self.eval_file(filename)

    def autreload(self):
        for filename in self.loaded_files:
            if self.loaded_files_mtime[filename] != os.path.getmtime(filename):
                print('Reloading ZmeiReactServer')
                self.reload_interpreter()
                break

    def evaljs(self, code, handle_errors=True, **kwargs):
        try:
            return self.jsi.evaljs(code, **kwargs)

        except JSRuntimeError as e:
            message = str(e)

            message = '\n' + colored('Error:', 'white', 'on_red') + ' ' + message

            print(message)
            m = re.search('\(line\s+([0-9]+)\)', message)
            if m:
                print('-' * 100)
                print('Source code:')
                print('-' * 100)
                row = int(m.group(1)) - 1
                source = code.splitlines()

                line = colored(source[row], 'white', 'on_red')
                print('\n'.join([f'{x+1}:\t{source[x]}' for x in range(max(0, row - 10), row)]))
                print(f'{row+1}:\t{line}')
                print('\n'.join([f'{x+1}:\t{source[x]}' for x in range(row + 1, min(row + 10, len(source) - 1))]))
                print('-' * 100)

    def load(self, filename):
        self.loaded_files_mtime[filename] = os.path.getmtime(filename)
        self.loaded_files.append(filename)
        self.eval_file(filename)

    def eval_file(self, filename):
        with open(filename) as f:
            self.evaljs(f)



#!/usr/bin/env python
from setuptools import find_packages, setup

project = "zmei"
version = "0.1.1"

setup(
    name=project,
    version=version,
    description="Intuitive web-framework, based on rock-solid opensource components.",
    author="Alex Rudakov",
    author_email="ribozz@gmail.com",
    url="https://github.com/Negative Space OU/zmei",
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
	'djangorestframework',
        'dukpy',
        'termcolor',
    ],
    dependency_links=[
    ],
    entry_points={
    },
    tests_require=[
        "djangorestframework",
        "coverage>=3.7.1",
        "mock>=1.0.1",
        "pytest",
    ],
)





from json import JSONEncoder

from django.db.models import QuerySet, Model
from django.utils.module_loading import import_string
from rest_framework import serializers


class DefaultModelSerializer(serializers.ModelSerializer):
    class Meta:
        model = None
        exclude = []


class ZmeiReactJsonEncoder(JSONEncoder):
    def __init__(self, *, view=None, skipkeys=False, ensure_ascii=True, check_circular=True, allow_nan=True, sort_keys=False,
                 indent=None, separators=None, default=None):
        super().__init__(skipkeys=skipkeys, ensure_ascii=ensure_ascii, check_circular=check_circular,
                         allow_nan=allow_nan, sort_keys=sort_keys, indent=indent, separators=separators,
                         default=default)
        if not view:
            raise AttributeError('ZmeiReactJsonEncoder: View is required')
        self.view = view

        self.serializers_cache = {}

    def create_default_serializer(self, model):
        serializer = type('_', (DefaultModelSerializer,), {})
        serializer.Meta.model = model

        return serializer

    def get_model_serializer(self, model):
        if model in self.serializers_cache:
            return self.serializers_cache[model]

        serializer = None
        try:
            index_import_path = f"{'.'.join(model.__module__.split('.')[:-1])}.serializers.index"
            serializer_index = import_string(index_import_path)

            if model in serializer_index:
                serializer = serializer_index[model].get('_')
        except ImportError:
            pass

        if not serializer:
            serializer = self.create_default_serializer(model)

        self.serializers_cache[model] = serializer

        return serializer

    def default(self, o):
        # url object
        if hasattr(o, '__name__') and o.__name__ == 'url':
            return self.view.kwargs

        if isinstance(o, QuerySet):
            serializer = self.get_model_serializer(o.model)
            return serializer(o, many=True).data

        if isinstance(o, Model):
            serializer = self.get_model_serializer(o)
            return serializer(o).data

        print(f'WARN: ZmeiReactJsonEncoder -> Do not know how to encode "{o.__class__}"', o)
        return None
